-- This Source Code Form is subject to the terms of the bCDDL, v. 1.1.
-- If a copy of the bCDDL was not distributed with this
-- file, You can obtain one at http://beamng.com/bCDDL-1.1.txt

local M = {}

local max = math.max
local min = math.min
local clamp = clamp
local abs = math.abs
local floor = math.floor
local sign = sign

local rpmToAV = 0.104719755
local avToRPM = 9.549296596425384
local torqueToPower = 0.0001404345295653085
local psToWatt = 735.499

local lastChargingState = {}
local socThresholdHigh = 0.9
local socThresholdLow = 0.7
local chargingPower = 500 --kW

local function engineStartSound()
  
end
local function engineStopSound()
  
end
local function engineRunSound()
  
end

local function startRecharge(energyStorageName)
  lastChargingState[energyStorageName] = 1
end
local function stopRecharge(energyStorageName)
  lastChargingState[energyStorageName] = 0
end

local function recharge(energyStorageName,energyStorageTable,dt)
  if energyStorageTable.energyType == "electricEnergy" then
    energyStorage.getStorage(energyStorageName).storedEnergy = energyStorageTable.storedEnergy + chargingPower*dt*1200 
  else
    energyStorage.getStorage(energyStorageName).storedEnergy = energyStorageTable.storedEnergy + chargingPower*dt*2620 --内燃机效率低，燃料消耗功率比电力消耗功率大
  end
end

local function updateEnergyStorage(energyStorageName,energyStorageTable,dt) --能量储存名称，此名称对应的表，时间增量
  if lastChargingState[energyStorageName] ~= 1 then 
    if energyStorageTable.storedEnergy <= energyStorageTable.energyCapacity * socThresholdLow then 
      startRecharge(energyStorageName)
    end 
  else
    if energyStorageTable.storedEnergy >= energyStorageTable.energyCapacity * socThresholdHigh then
      stopRecharge(energyStorageName)
    else
      recharge(energyStorageName,energyStorageTable,dt)
    end

  end
  
end

local function updateGFX(dt)
  for _,v in pairs(lastChargingState)do
    updateEnergyStorage(_,energyStorage.getStorage(_), dt) --能量储存名称，此名称对应的表，时间增量
  end

  --guihooks.message({txt = "mainBattery" .." storage:" .. energyStorage.getStorage("mainBattery").storedEnergy , context = {ttl = 3, color = "info"}})
  --guihooks.message({txt = "mainTank" .." storage:" .. energyStorage.getStorage("mainTank").storedEnergy , context = {ttl = 3, color = "info"}})
  --guihooks.message({txt = "lastChargingState:" .. lastChargingState.mainBattery , context = {ttl = 3, color = "info"}})
end

local function reset()
  for _, v in pairs(lastChargingState)do
    lastChargingState[_] = 0
  end
end

local function init(jbeamData)
  for  _, v in pairs(energyStorage.getStorages())do -- for (元素名), (元素内容) in pairs(表) do ... end
    if v ~= nil and v.energyCapacity > 0 then
      lastChargingState[_] = 0
    end
  end
  
  socThresholdHigh = jbeamData.socThresholdHigh or 0.9
  socThresholdLow = jbeamData.socThresholdLow or 0.7
  chargingPower = jbeamData.chargingPower or 500

  if socThresholdHigh > 1 then socThresholdHigh = 1 end --预防下限大于上限和超出soc极限带来的bug
  if socThresholdLow < 0 then socThresholdLow = 0 end
  if socThresholdHigh < socThresholdLow + 0.005 then
    if socThresholdHigh - 0.05 >= 0 then
      socThresholdLow = socThresholdHigh - 0.05
    elseif socThresholdLow + 0.05 <= 1 then
      socThresholdHigh = socThresholdLow + 0.05
    else
      socThresholdHigh = 0.9
      socThresholdLow = 0.7
    end
  end
end

M.init = init
M.reset = reset
M.updateGFX = updateGFX

return M
