-- This Source Code Form is subject to the terms of the bCDDL, v. 1.1.
-- If a copy of the bCDDL was not distributed with this
-- file, You can obtain one at http://beamng.com/bCDDL-1.1.txt

local M = {}
M.type = "auxilliary"
M.defaultOrder = 1100
M.name = jatoDynamicDownforce

local max, min, abs = math.max, math.min, math.abs


local lastjatoDynamicDownforceInput = 0


local thrusterCount = 0
local thrusterNodes

local smokeTick = 0

local thrusterLoopName
local thrusterLoops1
local thrusterLoops2


--                                               (id               , uiName, icon             , color   , ringValue    , onClickCallback                                                     , remove)
local function updateSimpleControlButtons()
  extensions.ui_simplePowertrainControl.setButton("jatoDynamicDownforce", "jatoDynamicDownforce", "powertrain_jatoDynamicDownforce", "9072bc", 0.8, "electrics.values.togglejatoDynamicDownforce = 1 - (electrics.values.togglejatoDynamicDownforce or 1)")
end

local function updateGFX(dt)
  --local jatoDynamicDownforceInput = min(electrics.values.jatoDynamicDownforceInput or 0, 1)--输入为空就转换成0，输入大于1就限位到1
  local throttleInput = 0
  throttleInput = min(input.throttle*1.7+input.brake*1.7+abs(input.steering*3.5)+0,1) or 0
  --local throttleInput = input.steering or 0
  if electrics.values.togglejatoDynamicDownforce==0 then
    throttleInput = 0
  end
  local jatoDynamicDownforceInput = throttleInput
  local gearIndex = electrics.values.gearIndex or 0

  --[[if throttleInput == 1 and gearIndex >= 0 and (input.state.throttle.filter == FILTER_PAD or input.state.throttle.filter == FILTER_DIRECT) then
    jatoDynamicDownforceInput = 1
  end--]]





  smokeTick = smokeTick + dt * jatoDynamicDownforceInput * 100
  if smokeTick > 1 then
    for i = 1, thrusterCount, 1 do
      local thruster = thrusterNodes[i]
      obj:addParticleByNodes(thruster[1], thruster[2], 20, 81, 0.01, 1)
    end
    smokeTick = 0
  end

  if jatoDynamicDownforceInput > 0 and lastjatoDynamicDownforceInput <= 0 then
    thrusterLoops1 = thrusterLoops1 or {}
    for i = 1, thrusterCount do
      if thrusterLoops2 and thrusterLoops2[i] then
        obj:setVolume(thrusterLoops2[i], 0)
      end
      thrusterLoops1[i] = thrusterLoops1[i] or obj:createSFXSource2(thrusterLoopName, "AudioDefaultLoop3D", "jatoDynamicDownforceThrusterLoop", thrusterNodes[i][1], 0)
      obj:setVolume(thrusterLoops1[i], jatoDynamicDownforceInput)
      obj:playSFX(thrusterLoops1[i])
    end
  end

  if jatoDynamicDownforceInput <= 0 and lastjatoDynamicDownforceInput > 0 then
    for i = 1, thrusterCount do
      obj:stopSFX(thrusterLoops1 and thrusterLoops1[i] or -1)
    end
    thrusterLoops1, thrusterLoops2 = thrusterLoops2, thrusterLoops1
  end



  electrics.values.jatoDynamicDownforce = jatoDynamicDownforceInput

  lastjatoDynamicDownforceInput = jatoDynamicDownforceInput


end

local function reset()


  smokeTick = 0

  for i = 1, thrusterCount do
    obj:stopSFX(thrusterLoops1 and thrusterLoops1[i] or -1)
    obj:stopSFX(thrusterLoops2 and thrusterLoops2[i] or -1)
  end

  electrics.values.jatoDynamicDownforceInput = 0
  electrics.values.jatoDynamicDownforce = 0
  electrics.values.togglejatoDynamicDownforce = 1
end

local function init(jbeamData)


  smokeTick = 0

  electrics.values.jatoDynamicDownforceInput = 0
  electrics.values.jatoDynamicDownforce = 0
  electrics.values.togglejatoDynamicDownforce = 1

  thrusterNodes = {}
  if jbeamData.thrusterNodes_nodes then
    local nodeCount = #jbeamData.thrusterNodes_nodes
    if nodeCount % 2 == 0 then
      for i = 1, nodeCount, 2 do
        table.insert(thrusterNodes, {jbeamData.thrusterNodes_nodes[i], jbeamData.thrusterNodes_nodes[i + 1]})
      end
    else
      log("E", "jatoDynamicDownforce.init", "Even number of thruster nodes are required!")
    end
  end

  thrusterCount = #thrusterNodes
  thrusterLoopName = jbeamData.thrusterLoopName or "event:>Vehicle>Thrusters"

  extensions.inputAnalyzer.registerInputString("uuddlrlrba", inputStringCallback)
end

M.init = init--从jbeam提取数据
M.reset = reset--
M.updateGFX = updateGFX --以帧生成时间为间隔更新数据
--M.update = updatePhysics --以1/2000秒为间隔更新数据

M.updateSimpleControlButtons = updateSimpleControlButtons--更新按键状态

return M
