-- This Source Code Form is subject to the terms of the bCDDL, v. 1.1.
-- If a copy of the bCDDL was not distributed with this
-- file, You can obtain one at http://beamng.com/bCDDL-1.1.txt

local M = {}
M.type = "auxilliary"
M.defaultOrder = 8000
M.relevantDevice = nil
local storedJbeamData--储存外部输入参数，供所有函数读取

M.scaledTransmissionGearRatios={}
M.scaledFinalDriveGearRatio={}
M.scaledCVTMaxGearRatio= {}

local timer= 0

local function gearRatiosCopy(origional)
    local copy
    if type(origional)=="table" then
        copy= {}
        for key, value in pairs(origional) do
            copy[key]= value
        end
    else
        copy= origional
    end
    return copy
end



local function scaleTransmissionGearRatios(deviceName)--缩放除cvt外的变速箱齿比
    if storedJbeamData.transmissionGearRatioAdjust then
        for key, value in pairs(M.scaledTransmissionGearRatios[deviceName]) do
            M.scaledTransmissionGearRatios[deviceName][key]= value* storedJbeamData.transmissionGearRatioAdjust
        end
    end
end

local function scaleCVTTransmissionGearRatios(deviceName)--缩放cvt齿比
    if storedJbeamData.transmissionGearRatioAdjust then
        M.scaledCVTMaxGearRatio[deviceName]= M.scaledCVTMaxGearRatio[deviceName]* storedJbeamData.transmissionGearRatioAdjust
    end
end

local function setTorqueConverterParameters(_,device)--覆写液力变矩器参数
    device.converterTorque =  math.huge
    --车辆选项里可能不会选这项，所以所有参数都得判断存不存在，不存在则不覆写
    --提升扭矩容量
    if storedJbeamData.converterDiameter then
        device.converterDiameter= storedJbeamData.converterDiameter
        device.kFactorCoef = device.fluidDensity * device.converterDiameter * device.converterDiameter * device.converterDiameter * device.converterDiameter * device.converterDiameter
    end
    --耦合曲线
    if storedJbeamData.couplingAVRatio then
        device.couplingAVRatio= storedJbeamData.couplingAVRatio
    end
    if storedJbeamData.stallTorqueRatio then
        device.stallTorqueRatio= storedJbeamData.stallTorqueRatio
    end
    if storedJbeamData.converterStiffness then
        device.converterStiffness= storedJbeamData.converterStiffness
    end
    --锁止离合器参数
    if storedJbeamData.lockupClutchTorque then
        device.lockupClutchTorque= storedJbeamData.lockupClutchTorque
    end
    --log('I',"transmissionPerformanceUpgrade",dumps(device.converterDiameter))
end

local function scaleFinalDriveGearRatio(_,device)
    if storedJbeamData.finalDriveGearRatioAdjust then
        device.gearRatio= device.gearRatio* storedJbeamData.finalDriveGearRatioAdjust
        if device.invGearRatio then
            device.invGearRatio= 1/device.gearRatio
        end
    end
end


local function reset()
    for _,device in pairs(powertrain.getDevices()) do
        if device.type=="torqueConverter" then
            setTorqueConverterParameters(_,device)
        elseif M.scaledTransmissionGearRatios[device.name] then --and (device.gearRatios ~= M.scaledTransmissionGearRatios[device.name]) then
            --scaleTransmissionGearRatios(_,device)
            device.gearRatios = gearRatiosCopy(M.scaledTransmissionGearRatios[device.name])
        elseif device.type=="cvtGearbox" then --and device.maxGearRatio~=M.scaledCVTMaxGearRatio then
            --scaleCVTTransmissionGearRatios(_,device)
            device.maxGearRatio = gearRatiosCopy(M.scaledCVTMaxGearRatio[device.name])
            device.minGearRatio= 0.1
        elseif M.scaledFinalDriveGearRatio[device.name] then --and device.gearRatio~=M.scaledFinalDriveGearRatio[device.name] then
            --scaleFinalDriveGearRatio(_,device)
            device.gearRatio = gearRatiosCopy(M.scaledFinalDriveGearRatio[device.name])
            if device.invGearRatio then
                device.invGearRatio= 1/device.gearRatio
            end
        end
    end
    log('I',"transmissionPerformanceUpgrade","scaled transmission gear ratios:"..dumps(M.scaledTransmissionGearRatios))
    log('I',"transmissionPerformanceUpgrade","scaled final drive gear ratio: "..dumps(M.scaledFinalDriveGearRatio))
    
    timer= 3000
end


local function init(jbeamData)
    storedJbeamData= jbeamData

    for _,device in pairs(powertrain.getDevices()) do
        if device.type=="automaticGearbox"or device.type=="dctGearbox"or device.type=="manualGearbox"or device.type=="sequentialGearbox" then
            M.scaledTransmissionGearRatios[device.name]= gearRatiosCopy(device.gearRatios)
            scaleTransmissionGearRatios(device.name)
            device.gearRatios= gearRatiosCopy(M.scaledTransmissionGearRatios[device.name])
        elseif device.type=="cvtGearbox" then
            device.minGearRatio= 0.1
            M.scaledCVTMaxGearRatio[device.name]= gearRatiosCopy(device.maxGearRatio)
            scaleCVTTransmissionGearRatios(device.name)
            device.maxGearRatio= gearRatiosCopy(M.scaledCVTMaxGearRatio[device.name])
            log('I',"transmissionPerformanceUpgrade","scaled CVT gear ratios: "..dumps(device.maxGearRatio).."~"..dumps(device.minGearRatio))
        elseif device.type=="torqueConverter" then
            setTorqueConverterParameters(_,device)
        elseif (device.type=="differential" or device.type=="torsionReactor" or device.type=="shaft") and device.gearRatio~=1 then
            scaleFinalDriveGearRatio(_,device)
            M.scaledFinalDriveGearRatio[device.name]= gearRatiosCopy(device.gearRatio)
        end
    end
    
    log('I',"transmissionPerformanceUpgrade","scaled transmission gear ratios:"..dumps(M.scaledTransmissionGearRatios))
    log('I',"transmissionPerformanceUpgrade","scaled final drive gear ratio: "..dumps(M.scaledFinalDriveGearRatio))
end

local function updateGFX(dt)
    if timer<= 2 then
        timer= timer+ dt
        return
    end
    timer= 0

    log('I',"transmissionPerformanceUpgrade","scaled transmission gear ratios:"..dumps(M.scaledTransmissionGearRatios))
    log('I',"transmissionPerformanceUpgrade","scaled final drive gear ratio: "..dumps(M.scaledFinalDriveGearRatio))
end

M.init = init
M.reset = reset
M.updateGFX = nop

return M